;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author : Gregor Mhler (moehler@ims.uni-stuttgart.de)     */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* Sets up the German lexicons and defines a addenda lexicon with sampa  */
;/* phones.                                                               */
;/*                                                                       */
;/*=======================================================================*/

(require 'ims_german_POS) ; for the pos-mapping

; if this parameter is 0 then glottal stops are inserted in the postlex rules
(Param.def 'lex_with_glottalstop 1)

;; this lexicons is based on Celex
;; It contains the iso-latin1 umlauts AND the ae..ss etc. variants.
(define (setup_gcelex_lex)
  (Param.set 'lex_with_glottalstop 0)
  (if (not (member_string "gcelex" (lex.list)))
      (begin
	(lex.create "gcelex")
	(lex.set.compile.file 
	 (string-append libdir "/german/dicts/gcelex-iso-sampa.out"))
	(lex.set.phoneset "german")
	(require 'ims_german_lts)
	(lex.set.lts.method 'lts_rules)
	(lex.set.lts.ruleset 'german)
	(lex.set.post_hooks map_pos_celex_ustts)
	(sampa_addenda))))

;; this lexicons is based on Bomp (lexicon from Univ. of Bonn)
;; It contains the iso-latin1 umlauts AND the ae..ss etc. variants.
(define (setup_bomp_lex)
  (Param.set 'lex_with_glottalstop 1)
  (if (not (member_string "bomp" (lex.list)))
      (begin
	(lex.create "bomp")
	(lex.set.compile.file 
	 (string-append libdir "/german/dicts/bomp_full.out"))
	(lex.set.phoneset "german_sampa")
	(require 'ims_german_lts)
	(lex.set.lts.method 'lts_rules)
	(lex.set.lts.ruleset 'german)
	(lex.set.post_hooks map_pos_bomp_ustts)
	(sampa_addenda))))


(define (sampa_addenda)
"(sampa_addenda) 
Addenda entries annotated with ustts-tags"
  (lex.add.entry '("unknown" ADV (((u: p) 1) ((s a:) 0)))) ;; the german version of 'unknown'
  (lex.add.entry '("Schrgstrich" N (((S R E k) 1) ((S t R I C) 0))))  
  (lex.add.entry '("Schraegstrich" N (((S R E k) 1) ((S t R I C) 0))))  
  (lex.add.entry '("Joule" N (((dZ u: l) 1))))
  (lex.add.entry '("Festival" N (((f E s) 1) ((t i) 0) ((v 9 l) 0))))
  (lex.add.entry '("Gregor" N (((g R e:) 1) ((g o: 6) 0))))
  (lex.add.entry '("Mhler" N (((m 2:) 1) ((l 6) 0))))
  (lex.add.entry '("Moehler" N (((m 2:) 1) ((l 6) 0))))
  (lex.add.entry '("Stefan" N (((S t 9) 1) ((f a: n) 0))))
  (lex.add.entry '("Annegret" N (((a) 1) ((n E) 0) ((g R e: t) 0))))
  (lex.add.entry '("Wolfgang" N (((v O l f) 1) ((g a N) 0))))
  (lex.add.entry '("Grzegorz" N (((k S 9) 1) ((g O R S) 0))))
  (lex.add.entry '("Dogil" N (((d o:) 1) ((g i: l) 0))))
  (lex.add.entry '("Judith" N (((j u:) 1) ((d I t) 0))))
  (lex.add.entry '("Isabel" N ((( I s @) 1) ((b 9 l) 0))))
  (lex.add.entry '("Stuttgart" N ((( S t U t ) 1) ((g a R t) 0))))
  (lex.add.entry '("Alan" N ((( E 6 ) 1)(( l @ n) 0))))
  (lex.add.entry '("Black" N ((( b l E k) 1))))
  (lex.add.entry '("Paul" N ((( p O l) 1))))
  (lex.add.entry '("Taylor" N ((( t e:) 1)((l O R) 0))))
  (lex.add.entry '("Edinburgh" N ((( E ) 1)((d I n) 0)((b 9 k) 1))))
  (lex.add.entry '("Diphon" N (((d i:) 1)((f o: n) 0))))
  (lex.add.entry '("Diphonbibliothek" N (((d i:) 1)((f o: n) 0)((b I) 0)((b l i:) 0)((o:) 0)((t e: k) 1))))
  (lex.add.entry '("Diphonstimme" N (((d i:) 1)((f o: n) 0)((S t I) 0)((m @) 0))))
  (lex.add.entry '("MBROLA" N ((( E m ) 0)((b R o:) 1)((l a) 0))))
  (lex.add.entry '("Mons" N (((m a~ s) 0))))
  (lex.add.entry '("Vorfhrung" N (((f o: 6) 1) ((f y:) 0) ((R U N) 0))))
  (lex.add.entry '("Vorfuehrung" N (((f o: 6) 1) ((f y:) 0) ((R U N) 0))))
  (lex.add.entry '("zusammengestpselt" V ((( t s u: )0) (( z a )1) (( m @ n )0)(( g @ )0) (( S t 9 p )1) (( s @ l t )0))))
  (lex.add.entry '("zusammengestoepselt" V ((( t s u: )0) (( z a )1) (( m @ n )0)(( g @ )0) (( S t 9 p )1) (( s @ l t )0))))
  (lex.add.entry '("genau" ADV (((g E) 0) ((n aU) 1))))
  (lex.add.entry '("email" N (((i:) 1) ((m e:) 0) ((I l) 0))))
  (lex.add.entry '("im" KO (((I m) 1))))
  (lex.add.entry '("am" KO (((a m) 1))))
  (lex.add.entry '("nicht" PTK (((n I C t) 1)))) ; for intonation we want 'nicht' special
  (lex.add.entry '("Jakob" N ((( j a ) 1)((k O p) 0))))
  (lex.add.entry '("Clinton" N ((( k l I n ) 1)((t @ n) 0))))
  (lex.add.entry '("sechzehn" CARD (((z E C) 1) ((ts e: n) 0))))
  (lex.add.entry '("Aids" N (((E: I t s) 1))))
  (lex.add.entry '("Willy" N (((v I) 1) ((l i:) 0))))
  (lex.add.entry '("Willi" N (((v I) 1) ((l i:) 0))))
  (lex.add.entry '("Israel" N (((I s) 1) ((R a:) 0) ((e: l) 0))))
  (lex.add.entry '("Israels" N (((I s) 1) ((R a:) 0) ((e: l s) 0))))
  (lex.add.entry '("Nachrichtenagentur" N (((n a: x) 1) ((r I C) 0) ((t @ n) 0) ((a) 0) ((g @ n) 0) ((t u: r) 1))))
  (lex.add.entry '("Interfax" N (((I n) 1) ((t 6) 0) ((f a: k s) 0))))
  (lex.add.entry '("Arizona" N (((a:) 0) (( R i:) 1) ((z o:) 0) ((n a) 0))))
  (lex.add.entry '("Vietnamesen" N (((v i:) 0) ((E t) 0) ((n a) 1) ((m e: ) 0) ((z @ n) 0))))
  (lex.add.entry '("Michael" N (((m I) 1) ((C a:) 0) ((e: l) 0))))
  (lex.add.entry '("Michaela" N (((m I) 1) ((C a:) 0) ((e: l) 0) ((a) 0))))
  (lex.add.entry '("Grand" N (((g R a^) 1))))
  (lex.add.entry '("Prix" N (((p R i:) 1))))
  (lex.add.entry '("Public" ADJ (((p a b) 1) ((l I k) 0))))
  (lex.add.entry '("Nordkorea" N (((n O R t) 1) ((k o:) 0) ((r e: a:) 0) ((a:) 0))))
  (lex.add.entry '("Korea" N (((k o:) 0) ((r e:) 1) ((a:) 0))))
  (lex.add.entry '("Idee" N (((I) 0) ((d e:) 1))))
  (lex.add.entry '("Miami" N (((m ai) 0) ((E) 1) ((m i:) 0))))
  (lex.add.entry '("Washington" N (((v O ) 1) ((S I N ) 0) ((t @ n) 0))))
  (lex.add.entry '("London" N (((l O n ) 1) ((d O n) 0))))
  (lex.add.entry '("Sunday" N (((s a n) 1) ((d eI) 0))))
  (lex.add.entry '("Times" N (((t ai m s) 1))))
  (lex.add.entry '("Perez" N (((p e:) 1) ((R E s) 0))))
  (lex.add.entry '("Elizabeth" N (((e:) 0) ((l i:) 1) ((z a:) 0) ((b e: t) 0))))
  (lex.add.entry '("zweiundzwanzigjaehrig" ADV (((ts v aI) 1)  ((U n t) 0)  ((ts v a n) 0)  ((ts I C) 0)  ((j E:) 1)  ((R I C) 0))))
  (lex.add.entry '("zweiundzwanzigjhrig" ADV (((ts v aI) 1)  ((U n t) 0)  ((ts v a n) 0)  ((ts I C) 0)  ((j E:) 1)  ((R I C) 0))))
  (lex.add.entry '("Cousine" N (((k U) 0) ((z i:) 1) ((n @) 0))))
  (lex.add.entry '("verbmobil" N (((v E R p) 1) ((m o: b) 0) ((i: l) 0))))
  (lex.add.entry '("Cannstatt" N (((k a n) 1) ((S t a t) 0))))
  (lex.add.entry '("Umsteigebahnhof" N (((U m) 1) ((S t aI) 0) ((g @) 0) ((b a: n) 0) ((h o: f) 0))))
  (lex.add.entry '("Feuerbach" N (((f OY) 1) ((6) 0) ((b a x) 0))))
  (lex.add.entry '("Feuersee" N (((f OY) 1) ((6) 0) ((z e:) 0))))
  (lex.add.entry '("Stadion" N (((S t a: d) 1) ((j o: n) 0))))
  (lex.add.entry '("Obertuerkheim" N (((o:) 0) ((b @) 0) ((t y: R k) 1) ((h aI m) 0))))
  (lex.add.entry '("Obertrkheim" N (((o:) 0) ((b @) 0) ((t y: R k) 1) ((h aI m) 0))))
  (lex.add.entry '("Oesterfeld" N (((2: s) 1) ((t 6) 0) ((f E l t) 0))))
  (lex.add.entry '("sterfeld" N (((2: s) 1) ((t 6) 0) ((f E l t) 0))))
  (lex.add.entry '("Schwabstrasse" N (((S v a p) 1) ((S t R a:) 0) ((s @) 0))))
  (lex.add.entry '("Schwabstrae" N (((S v a p) 1) ((S t R a:) 0) ((s @) 0))))
  (lex.add.entry '("Sommerrain" N (((z O) 1) ((m 6) 0) ((R aI n) 0))))
  (lex.add.entry '("Untertuerkheim" N (((U n) 0) ((t @) 0) ((t y: R k) 1) ((h aI m) 0))))
  (lex.add.entry '("Untertrkheim" N (((U n) 0) ((t @) 0) ((t y: R k) 1) ((h aI m) 0))))
  (lex.add.entry '("Vaihingen" N (((f aI) 1) ((h I N) 0) ((@ n) 0))))
  (lex.add.entry '("Zuffenhausen" N (((t s U) 0) ((f E n) 0) ((h aU) 0) ((s @ n) 0))))
  (lex.add.entry '("Zielort" N (((ts i: l) 1) ((? O R t) 0))))
  (lex.add.entry '("Beispiel" N (((b aI) 1) ((S p i: l) 0))))
  (lex.add.entry '("Militrexperten" N (((m i:) 0) ((l i:) 0) ((t E: R) 1) ((E k s) 0) ((p E R) 0) ((t @ n) 0))))
  (lex.add.entry '("Funktion" N (((f U N k) 0) ((ts j o: n) 1))))
  (lex.add.entry '("Kreditkartenfunktion" N (((k R E) 0) ((d i: t) 1) ((k a R) 0) ((t @ n) 0) ((f U N k) 0) ((ts j o: n) 0))))
  (lex.add.entry '("Oberlandesgericht" N (((o:) 0) ((b 6) 0) ((l a n) 1) ((d E s) 0) ((g @) 0) ((r I C t) 0))))
  (lex.add.entry '("Citybank" N (((s I) 1) ((t i:) 0) ((b a N k) 0))))
  (lex.add.entry '("Zugabeverordnung" N (((ts u:) 1)((g a:) 0)((b @) 0)((f 6) 0)((O R d) 0)((n U N) 0))))
  (lex.add.entry '("Bundesgerichtshof" N (((b U n) 0) ((d E s) 0) ((g @) 0) ((r I C t s) 1) ((h o: f) 0))))
  (lex.add.entry '("Dayton" N (((d e:) 1)((t @ n) 0))))
  (lex.add.entry '("Burns" N (((b 2: R n s) 1))))
  (lex.add.entry '("Version" N (((v E R) 0) ((s j o: n) 0))))
  (lex.add.entry '("painte" N (((p e: n) 1) ((t @) 0))))
  (lex.add.entry '("gesprochenem" ADJ (((g @) 0) ((S p R O x) 1) ((@ n) 0) ((@ m) 0))))
  (lex.add.entry '("Informationsverarbeitung" N  (((I n) 0) ((f O R) 0) ((m a) 0) ((ts j o: n s) 1) ((f 6) 0) ((a R) 0) ((b aI) 0) ((t U N) 0))))
  (lex.add.entry '("Informationsfluten" N (((I n) 0) ((f O R) 0) ((m a) 0) ((ts j o: n s) 1) ((f l u:) 0) ((t @ n) 0))))
  (lex.add.entry '("geschriebenem" ADJ (((g @) 0) ((S R i: b) 1) ((@ n) 0) ((@ m) 0))))
  (lex.add.entry '("zur" APP (((ts U 6) 1))))
  (lex.add.entry '("Interaktion" N (((I n t) 0) ((6) 0) ((a k) 0) ((t s j o: n) 1))))
  (lex.add.entry '("Generierung" N (((g @) 0) ((n @) 0) ((R i:) 1) ((R U N) 0))))
  (lex.add.entry '("Wissensgesellschaft" N (((v I s) 1) ((@ n s) 0) ((g @ z) 0) ((@ l S) 0) ((a f t) 0))))
  (lex.add.entry '("Benutzerschnittstellen" N (((b @) 0) ((n U) 1) ((ts 6) 0) ((S n I t) 0) ((S t E l) 0) ((@ n) 0))))
  (lex.add.entry '("Schnittstellen" N (((S n I t) 0) ((S t E l) 0) ((@ n) 0))))
  (lex.add.entry '("Kommunikationsstil" N (((k O) 0) ((m U) 0) ((n I) 0) ((k a) 0) ((ts j o: n s) 1) ((S t i: l) 0))))
  (lex.add.entry '("Computerlaien" N (((k O m) 0) ((p j u: t) 1) ((6) 0) ((l aI) 0) ((@ n) 0))))
  (lex.add.entry '("Bedienkonzepten" N (((b @) 0) ((d i: n) 1) ((k O n) 0) ((ts E p) 0) ((t @ n) 0))))
  (lex.add.entry '("Internet" N (((I n) 1) ((t 6) 0) ((n E t) 0))))
  (lex.add.entry '("Kommunikationsnetze" N (((k O) 0) ((m u:) 0) ((n I) 0) ((k a) 0) ((ts j o: n s) 1)  ((n E t) 0)  ((s @) 0))))
  (lex.add.entry '("bestehenden" ADJ (((b @) 0) ((S t e:) 1) ((@ n) 0) ((d @ n) 0))))
  (lex.add.entry '("Multimedia" N (((m U l) 0) ((t I) 0) ((m e:) 1) ((d I) 0) ((a) 0))))
  (lex.add.entry '("Mozarella" N (((m O) 0) ((ts a ) 0) ((R E) 1) ((l a) 0))))
  (lex.add.entry '("Margerita" N (((m a: R) 0) ((g a:) 0) ((R i:) 1) ((t a:) 0))))
  (lex.add.entry '("Colada" N (((k o:) 1) ((l a:) 1) ((d a) 0))))
  (lex.add.entry '("Orangensaft" N (((o:) 0) ((R a N) 1) ((Z @ n) 0) ((z a f t) 0))))
  (lex.add.entry '("Hefe" N (((h e: f) 1) ((f @) 0))))
  (lex.add.entry '("Andreas" N (((a n) 0) ((d R e:) 1) ((a s) 0))))
  (lex.add.entry '("Smartkom" N (((s m a R t) 1) ((k O m) 0))))  
  (lex.add.entry '("Informationen" N (((I n) 0) ((f O 6) 0) ((m a) 0) ((ts j o:) 1) ((n @ n) 0))))
  (lex.add.entry '("Katastrophenfilm" N (((k a) 0) ((t a s) 0) ((t R o:) 1) ((f @ n) 0) ((f I l m) 0))))
  (lex.add.entry '("Komdie" N (((k O) 0) ((m 2:) 1) ((d I) 0) ((j @) 0))))
  (lex.add.entry '("Komdien" N (((k O) 0) ((m 2:) 1) ((d I) 0) ((j @ n) 0))))
  (lex.add.entry '("Odeon" N (((o:) 0) ((d e:) 1) ((j O n) 0) )))
  (lex.add.entry '("Bestuhlungsplan" N (((b @) 0) ((S t u:) 1) ((l U N s) 0) ((p l a: n) 0))))
  (lex.add.entry '("Stadion" N (((S t a:) 1) ((d i:) 0) ((o: n) 0))))
  (lex.add.entry '("Quatsch" N (((k w a tS) 0))))
  (lex.add.entry '("Sable" N (((s e: I b l) 1))))
  (lex.add.entry '("Utterance" N (((a) 1) ((t 6) 0) ((R E n ts) 0))))
  (lex.add.entry '("Chunk" N (((tS a N k) 1))))
  (lex.add.entry '("Chunks" N (((tS a N k s) 1))))
  (lex.add.entry '("Label" N (((l e: I b l) 1))))
  (lex.add.entry '("Labels" N (((l e: I b l s) 1))))
  (lex.add.entry '("Default" N (((d i:) 1) ((f O l t) 0))))
  (lex.add.entry '("Milena" N (((m i:) 0) ((l e:) 1) ((n a:) 0))))
  (lex.add.entry '("France" N (((f R a^ s) 1))))
  (lex.add.entry '("Wien" N (((v i: n) 1))))
  (lex.add.entry '("Synthesesystem" N
   (((z Y n) 0) ((t e:) 1) ((s @) 0) ((z Y s) 0) ((t e: m ) 0))))             
  (lex.add.entry '("Synthesesystems" N
   (((z Y n) 0) ((t e:) 1) ((s @) 0) ((z Y s) 0) ((t e: m s) 0))))
  (lex.add.entry '("Informationssystem" N (((I n) 0) ((f O R) 0) ((m a) 0) ((ts j o: n s) 1) ((s Y s) 0) ((t e: m) 0))))
  (lex.add.entry '("Heidelberger" ADJ (((h aI) 1) ((d @ l) 0) ((b E 6) 0) ((g 6) 0))))
  (lex.add.entry '("Reservierungsnummer" N (((R e:) 0) ((z E 6) 0) ((v i:) 1) ((R U N s) 0) ((n U) 0) ((m 6) 0))))
  (lex.add.entry '("Vorstellungsbeginn" N (((f o: 6) 1) ((S t E) 0) ((l U N s) 0) ((b @) 0) ((g I n) 0))))
  (lex.add.entry '("SmartKom" N (((s m a: 6 t) 1) ((k O m) 0))))
  (lex.add.entry '("prosodische" nil (((p R o:) 0) ((z o:) 1) ((d I) 0) ((S @) 0))))
  (lex.add.entry '("varianten" ADJ (((v a R) 0) ((j a n) 1) ((t @ n) 0))))
  (lex.add.entry '("Story" N (((S t O) 1) ((R i:) 0))))
  (lex.add.entry '("Militaersequenzen" N (((m i:) 0)  ((l I) 0)  ((t E R) 1)  ((z E) 0)  ((k v E n) 0)  ((ts @ n) 0))))
  (lex.add.entry '("Militrsequenzen" N (((m i:) 0)  ((l I) 0)  ((t E R) 1)  ((z E) 0)  ((k v E n) 0)  ((ts @ n) 0))))
  (lex.add.entry '("Smartakus" N (((s m a R) 1) ((t a:) 0) ((k u s) 0))))
)

(provide 'ims_german_lexicons)






