;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*             Date   :  September 1997                                  */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file contains auxiliary functions to deal with abbreviations,    */
;/* numbers, currencies, titles, months, and the definitions of           */
;/* ``german_currency'' from ``ger_abbr_currency_tab'',                   */
;/* ``german_months'' from ``ger_abbr_months_tab'' and                    */
;/* ``german_days'' from ``ger_abbr_days_tab''.                           */
;/*                                                                       */
;/*=======================================================================*/

; minor changes (roman numbers). Ursula Vollmer [21082000]
; added (url->list). Martin Barbisch [16112000]

(require 'ims_siodfuncs)
(require 'ims_german_token_to_words_lists)

(define (german_parse_1char name)
  "(german_parse_1char NAME) 
Returns a string containing the german word for the single char NAME
e.g. (german_parse_1char \":\")"
  (let ((expansion (ger_abbr_lookup name ger_char_tab)))
    (if expansion
	expansion
	(if (string-equal name "")
	    (list "scharfes" "s")
	    (if (string-matches name "[A-Z]")
		(list name) 
		(if (string-matches name "[a-z]")
		    (list name) 
		    (list "unknown")))))))	

(define (german_parse_charlist name)
  "(german_parse_charlist NAME) returns a list of words with the spoken sequence of chars"
  (cond 
   ((eq? (string-length name) 0) (list nil))
   ((eq? (string-length name) 1) (german_parse_1char(string-car name)))
   (t
    (append (german_parse_1char(string-car name))
	    (german_parse_charlist (string-cdr name))))))



(define (splice_string name sep)
  "(splice_string NAME SEP) 
   kicks out SEP and splices the rest of NAME."
  (list->string (removerec 
		 (car (string->list sep)) 
		 (string->list name))))
;  (let ((regexp (string-append ".*" sep ".*")))
;    
;    (list (string-append (symbol->string(string-before name sep))
;		(symbol->string(string-after name sep))))))

(define (split_string name sep)
  "(split_string NAME SEP)
   seperates the string NAME at SEP into a list of 2 words"
  (let ((regexp (string-append ".*" sep ".*")))
    
    (append (list (symbol->string(string-before name sep)))
	    (list (symbol->string(string-after name sep))))))

(define (sep_string name sep sepword)
  "(sep_string NAME SEP SEPWORD) seperates the string NAME at SEP 
into a list of 2 words seperated by sepword"
  (let ((regexp (string-append ".*" sep ".*")))
    
    (append (list (symbol->string(string-before name sep)))
	    (list sepword)
	    (list (symbol->string(string-after name sep))))))

;(define (german_parse_ordnungszahl number sex)
;  "(german_parse_ordnungszahl NUMBER SEX) Returns a list of strings
;    containing the german words for the stellenzahl NUMBER 
;    (german_parse_ordnungszahl 21) -> (ein und zwanzigste) it uses
;    german"
  
;  (let ((res_list (german_parse_cardinal number)))
;    (let ((last_word (car (last res_list)))
;	  (dmor_string nil))
      
;      (cond 
;       ((string-matches last_word "eins")	 (set! last_word '("erst")))
;       ((string-matches last_word "zwei")	 (set! last_word '("zweit")))
;       ((string-matches last_word "drei")	 (set! last_word '("dritt")))
;       ((string-matches last_word "vier")	 (set! last_word '("viert")))
;       ((string-matches last_word "fnf")	 (set! last_word '("fnft")))
;       ((string-matches last_word "sechs")	 (set! last_word '("sechst")))
;       ((string-matches last_word "sieben")	 (set! last_word '("siebt")))
;       ((string-matches last_word "acht")	 (set! last_word '("acht")))
;       ((string-matches last_word "neun")	 (set! last_word '("neunt")))
;       ((string-matches last_word "zehn")	 (set! last_word '("zehnt")))
;       ((string-matches last_word "elf")	 (set! last_word '("elft")))
;       ((string-matches last_word "zwlf")	 (set! last_word '("zwlft")))
;       ((string-matches last_word "dreizehn")	 (set! last_word '("dreizehnt")))
;       ((string-matches last_word "vierzehn")	 (set! last_word '("vierzehnt")))
;       ((string-matches last_word "fnfzehn")	 (set! last_word '("fnfzehnt")))
;       ((string-matches last_word "sechzehn")	 (set! last_word '("sechzehnt")))
;       ((string-matches last_word "siebzehn")	 (set! last_word '("siebzehnt")))
;       ((string-matches last_word "achtzehn")	 (set! last_word '("achtzehnt")))
;       ((string-matches last_word "neunzehn")	 (set! last_word '("neunzehnt")))
;       ((string-matches last_word "zwanzig")	 (set! last_word '("zwanzigst")))
;       ((string-matches last_word "dreiig")	 (set! last_word '("dreiigst")))
;       ((string-matches last_word "vierzig")	 (set! last_word '("vierzigst")))
;       ((string-matches last_word "fnfzig")	 (set! last_word '("fnfzigst")))
;       ((string-matches last_word "sechzig")	 (set! last_word '("sechzigst")))
;       ((string-matches last_word "siebzig")	 (set! last_word '("siebzigst")))
;       ((string-matches last_word "achtzig")    (set! last_word '("achtzigst")))
;       ((string-matches last_word "neunzig")	 (set! last_word '("neunzigst")))
;       ((string-matches last_word "hundert")	 (set! last_word '("hundertst")))
;       ((string-matches last_word "Tausend")	 (set! last_word '("Tausendst")))
;       ((string-matches last_word "tausend")	 (set! last_word '("tausendst")))
;       ((string-matches last_word "million")	 (set! last_word '("millionst")))
;       ((string-matches last_word "milliarden") (set! last_word '("milliardst")))
;       ((string-matches last_word "billion")	 (set! last_word '("billionst")))
;       ((string-matches last_word "billiarden") (set! last_word '("billiardst")))
;       ((string-matches last_word "trillionen") (set! last_word '("trilliardst")))
;       (t nil))
;      (print last_word)   
;      (set! dmor_string (list (string-append (car last_word) sex)  'g))
;      (print dmor_string)   
;      (set! last_word 
;	    (symbol->string
;	     (strcar
;	      (symbol->string 
;	       (string-after 
;		(car 
;		 (dmor.infl dmor_string)) " ")))))
;      (print last_word)   
;      (if (< (length last_word) 2)	;if dmor.infl returned nil or ("") something gone wrong
;	  res_list			;so return as mutch as possible
;	  (append (remove_last res_list) (list last_word))))))

(define (german_parse_ordinal numberstring suffix)
  "(german_parse_ordinal NUMBERSTRING SUFFIX) Returns a list of strings
    containing the german words for the stellenzahl NUMBERSTRING 
    (german_parse_ordinal 21) -> (ein und zwanzigste) it uses
    german"
  
  (let ((res_list (german_parse_cardinal numberstring)))
    (if (equal? res_list '(unknown))  '(unknown) 
    (let ((last_word (car (last res_list)))
	  (dmor_string nil))
      
      (cond 
       ((string-matches last_word "null")	(set! last_word '("nullt")))
       ((string-matches last_word "eins")	(set! last_word '("erst")))
       ((string-matches last_word "zwei")	(set! last_word '("zweit")))
       ((string-matches last_word "drei")	(set! last_word '("dritt")))
       ((string-matches last_word "vier")	(set! last_word '("viert")))
       ((string-matches last_word "fnf")	(set! last_word '("fnft")))
       ((string-matches last_word "sechs")	(set! last_word '("sechst")))
       ((string-matches last_word "sieben")	(set! last_word '("siebt")))
       ((string-matches last_word "acht")	(set! last_word '("acht")))
       ((string-matches last_word "neun")	(set! last_word '("neunt")))
       ((string-matches last_word "zehn")	(set! last_word '("zehnt")))
       ((string-matches last_word "elf")	(set! last_word '("elft")))
       ((string-matches last_word "zwlf")	(set! last_word '("zwlft")))
       ((string-matches last_word "dreizehn")	(set! last_word '("dreizehnt")))
       ((string-matches last_word "vierzehn")	(set! last_word '("vierzehnt")))
       ((string-matches last_word "fnfzehn")	(set! last_word '("fnfzehnt")))
       ((string-matches last_word "sechzehn")	(set! last_word '("sechzehnt")))
       ((string-matches last_word "siebzehn")	(set! last_word '("siebzehnt")))
       ((string-matches last_word "achtzehn")	(set! last_word '("achtzehnt")))
       ((string-matches last_word "neunzehn")	(set! last_word '("neunzehnt")))
       ((string-matches last_word "zwanzig")	(set! last_word '("zwanzigst")))
       ((string-matches last_word "dreiig")	(set! last_word '("dreiigst")))
       ((string-matches last_word "vierzig")	(set! last_word '("vierzigst")))
       ((string-matches last_word "fnfzig")	(set! last_word '("fnfzigst")))
       ((string-matches last_word "sechzig")	(set! last_word '("sechzigst")))
       ((string-matches last_word "siebzig")	(set! last_word '("siebzigst")))
       ((string-matches last_word "achtzig")	(set! last_word '("achtzigst")))
       ((string-matches last_word "neunzig")	(set! last_word '("neunzigst")))
       ((string-matches last_word "hundert")	(set! last_word '("hundertst")))
       ((string-matches last_word "Tausend")	(set! last_word '("Tausendst")))
       ((string-matches last_word "tausend")	(set! last_word '("tausendst")))
       ((string-matches last_word "million")	(set! last_word '("millionst")))
       ((string-matches last_word "milliarden")	(set! last_word '("milliardst")))
       ((string-matches last_word "billion")	(set! last_word '("billionst")))
       ((string-matches last_word "billiarden")	(set! last_word '("billiardst")))
       ((string-matches last_word "trillionen")	(set! last_word '("trilliardst")))
       (t nil))
      (set! last_word (string-append (car last_word) suffix))
    ;;  (print last_word) 
      (append (remove_last res_list) (list last_word))))))


(define (german_parse_fractal numberstring)
  "(german_parse_fractal NUMBERSTRING) returns the number NUMBERSTRING as
   a german fractal, example (german_parse_fractal 123)
   -> (ein hundert drei und zwanzigstel)"
  (let ((erglist (german_parse_cardinal numberstring))) ; unfotunately there is no let* in 
    (if (equal? erglist '(unknown))  '(unknown) 
	(let ((last_word (car (last erglist))))    ; fesivals SIOD
	  (cond 
	   ((string-matches last_word "eins")       (set! last_word '("eintel")))
	   ((string-matches last_word "zwei")       (set! last_word '("halbe")))
	   ((string-matches last_word "drei")       (set! last_word '("drittel")))
	   ((string-matches last_word "vier")       (set! last_word '("viertel")))
	   ((string-matches last_word "fnf")       (set! last_word '("fnftel")))
	   ((string-matches last_word "sechs")      (set! last_word '("sechstel")))
	   ((string-matches last_word "sieben")     (set! last_word '("siebtel")))
	   ((string-matches last_word "acht")       (set! last_word '("achtel")))
	   ((string-matches last_word "neun")       (set! last_word '("neuntel")))
	   ((string-matches last_word "zehn")       (set! last_word '("zehntel")))
	   ((string-matches last_word "elf")        (set! last_word '("elftel")))
	   ((string-matches last_word "zwlf")      (set! last_word '("zwlftel")))
	   ((string-matches last_word "dreizehn")   (set! last_word '("dreizehntel")))
	   ((string-matches last_word "vierzehn")   (set! last_word '("vierzehntel")))
	   ((string-matches last_word "fnfzehn")   (set! last_word '("fnfzehntel")))
	   ((string-matches last_word "sechzehn")   (set! last_word '("sechzehntel")))
	   ((string-matches last_word "siebzehn")   (set! last_word '("siebzehntel")))
	   ((string-matches last_word "achtzehn")   (set! last_word '("achtzehntel")))
	   ((string-matches last_word "neunzehn")   (set! last_word '("neunzehntel")))
	   ((string-matches last_word "zwanzig")    (set! last_word '("zwanzigstel")))
	   ((string-matches last_word "dreiig")    (set! last_word '("dreiigstel")))
	   ((string-matches last_word "vierzig")    (set! last_word '("vierzigstel")))
	   ((string-matches last_word "fnfzig")    (set! last_word '("fnfzigstel")))
	   ((string-matches last_word "sechzig")    (set! last_word '("sechzigstel")))
	   ((string-matches last_word "siebzig")    (set! last_word '("siebzigstel")))
	   ((string-matches last_word "achtzig")    (set! last_word '("achtzigstel")))
	   ((string-matches last_word "neunzig")    (set! last_word '("neunzigstel")))
	   ((string-matches last_word "hundert")    (set! last_word '("hundertstel")))
	   ((string-matches last_word "Tausend")    (set! last_word '("Tausendstel")))
	   ((string-matches last_word "tausend")    (set! last_word '("tausendstel")))
	   ((string-matches last_word "Million")    (set! last_word '("Millionstel")))
	   ((string-matches last_word "million")    (set! last_word '("millionstel")))
	   ((string-matches last_word "Millionen")  (set! last_word '("Millionstel")))
	   ((string-matches last_word "millionen")  (set! last_word '("millionstel")))
	   ((string-matches last_word "milliarden") (set! last_word '("milliardstel")))
	   ((string-matches last_word "billion")    (set! last_word '("billionstel")))
	   ((string-matches last_word "billiarden") (set! last_word '("billiardstel")))
	   ((string-matches last_word "trillionen") (set! last_word '("trilliardstel")))
	   (t nil))
	  (append (remove_last erglist) last_word)))))



(define (remove_last liste)
  (reverse (cdr (reverse liste))))

(define (german_fetch_currency token name)
  "(german_fetch_currency TOKEN NAME)
   returns the currency of the money-unit following NAME: If it is an 
   abbreviation stored in ger_abbr_currency_tab"
  (let ((curr (item.feat token "n.name")))
    (let 
	((expand 
	  (ger_abbr_lookup curr ger_abbr_currency_tab)))
      (if expand
	  (begin
	    expand)
	  (list curr)))))


(define (german_fetch_currency2 name)
  "(german_fetch_currency2 NAME)
   returns the currency of the money-unit following NAME: If it is an 
   abbreviation stored in ger_abbr_currency_tab"
  (let ((curr name))
    (let 
	((expand 
	  (ger_abbr_lookup curr ger_abbr_currency_tab)))
      (if expand
	  (begin
	    expand)
	  (list curr)))))


(define (ger_tok_roman_to_numstring roman)
  "(ger_tok_roman_to_numstring ROMAN)
Takes a string of roman numerals and converts it to a number and
then returns the printed string of that."
  (let ((val 0) (chars (symbolexplode roman)))
    (while chars
     (cond
      ((equal? (car chars) 'M)
       (set! val (+ 1000 val)))
      ((equal? (car chars) 'D)
       (set! val (+ 500 val)))
      ((equal? (car chars) 'C)
       (cond
	((equal? (car (cdr chars)) 'D)
	 (set! val (+ 400 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'M)
	 (set! val (+ 900 val))
	 (set! chars (cdr chars)))
	(t
       (set! val (+ 100 val)))))
      ((equal? (car chars) 'L)
       (cond
	((equal? (car (cdr chars)) 'D)
	 (set! val (+ 450 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'M)
	 (set! val (+ 950 val))
	 (set! chars (cdr chars)))
	(t
       (set! val (+ 50 val)))))
      ((equal? (car chars) 'X)
       (cond
	((equal? (car (cdr chars)) 'L)
	 (set! val (+ 40 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'C)
	 (set! val (+ 90 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'D)
	 (set! val (+ 490 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'M)
	 (set! val (+ 990 val))
	 (set! chars (cdr chars)))
	(t
       (set! val (+ 10 val)))))
      ((equal? (car chars) 'V)
       (cond
	((equal? (car (cdr chars)) 'L)
	 (set! val (+ 45 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'C)
	 (set! val (+ 95 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'D)
	 (set! val (+ 495 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'M)
	 (set! val (+ 995 val))
	 (set! chars (cdr chars)))
	(t
       (set! val (+ 5 val)))))
      ((equal? (car chars) 'I)
       (cond
	((equal? (car (cdr chars)) 'V)
	 (set! val (+ 4 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'X)
	 (set! val (+ 9 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'L)
	 (set! val (+ 49 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'C)
	 (set! val (+ 99 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'D)
	 (set! val (+ 499 val))
	 (set! chars (cdr chars)))
	((equal? (car (cdr chars)) 'M)
	 (set! val (+ 999 val))
	 (set! chars (cdr chars)))
	(t
	 (set! val (+ 1 val))))))
     (set! chars (cdr chars)))
    (format nil "%d" val)))


(define (ger_find_month_from_number token string-number)
  "(ger_find_month_from_number token string-number)
Find the textual representation of the month from the given string number"
  (let ((nnum (parse-number string-number)))
    (cond
     ((equal? 1 nnum)  (list "Januar"))
     ((equal? 2 nnum)  (list "Februar"))
     ((equal? 3 nnum)  (list "Mrz"))
     ((equal? 4 nnum)  (list "April"))
     ((equal? 5 nnum)  (list "Mai"))
     ((equal? 6 nnum)  (list "Juni"))
     ((equal? 7 nnum)  (list "Juli"))
     ((equal? 8 nnum)  (list "August"))
     ((equal? 9 nnum)  (list "September"))
     ((equal? 10 nnum) (list "Oktober"))
     ((equal? 11 nnum) (list "November"))
     ((equal? 12 nnum) (list "Dezember"))
     (t
      (cons "Monat"
	    (german_token_to_words token string-number))))))
      


;---[ url->list(_german) ]------------------------------------------------------

(define (url->list_german url)
  "(url->list_german URL)
   Calls url->list and inserts 't', 'Doppelpunkt' etc.
   That is, it converts the string URL 
    (e.g. \"http://www.abc.de/fgh\" or \"myname@ims.uni-stuttgart.de\")
   to a list 
    ((\"http\" \"Doppelpunkt\" \"Doppelslsch\" \"www\" \"abc\" \"de\" \"fgh\")
     or (\"myname\" \"t\" \"ims\" \"uni-stuttgart\" \"de\"))."

  (let ((ulist) (strbs "") (sep "://"))

    ; URL or EMAIL?
    (mapcar 
     (lambda (elem)  (if (string-equal elem "@") (set! sep "@")))
     (string->list url)
    )

    (set! strbs (string-before url sep))

    ; with or without 'http://', 'myname@', etc. ?
    (if (string-equal strbs "")
	(set! ulist '())
	(begin
	  (if (string-equal sep "@")
	      (set! ulist (list strbs "t"))
	      (set! ulist (list strbs "Doppelpunkt" "Doppelslsch"))
          )
	  (set! url (string-after url sep))
        )
    )

    ; complete the list (and return it to caller)
    (set! ulist (append ulist (url->list url)))
  )
)


(define (url->list url)
  "(url->list URL)
   Converts the string URL 
    (e.g. \"http://www.abc.de/fgh\" or \"myname@ims.uni-stuttgart.de\")
   to a list 
    ((\"http\" \"www\" \"abc\" \"de\" \"fgh\")
     or (\"myname\" \"ims\" \"uni-stuttgart\" \"de\")).
   It separates URL at '@', ':', '/', '\\' and '.'."

  (let ((ulist) (str ""))

    ; make the list, separate by '.', ':', '\' and '/'
    (mapcar 
     (lambda (elem)
       (if (or (string-equal elem ".")  (string-equal elem ":")
               (string-equal elem "\\") (string-equal elem "/")
	       (string-equal elem "@"))
	   (begin
	     (if (not (string-equal str ""))
		 (set! ulist (append ulist (list str))))
	     (set! str   "")
	   )
	   (set! str (string-append str elem))
       )
     )
     (string->list (string-append url "."))
    )
    ulist
  )
)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; the following functions have all to do with abbreviations!!!!



;; for bigger tables we need hash-table-functionality.  most
;; lisp-dialects support them but SIOD don't.  these functions have to
;; be rewritten and the above assoc lists has to be converted to
;; hash-tables if in future-versions we want work to be domne by
;; hashtables.


(define (ger_abbr_lookup abbr tab)
  "(ger_abbr_lookup ABBR TAB)
   looks up an ABBR in a assoc-list TAB. 
   for bigger tabs we should use hash-tables"
  (cdr (assoc abbr tab )))

(define (ger_abbr_getkeys tab)
  "(ger_abbr_getkeys TAB)
  returns a list of all keys in the assoc-list TAB"
  (if (eq tab nil)
      nil
      (cons (caar tab) (ger_abbr_getkeys (cdr tab)))))

(define (ger_abbr_getvals tab)
  "(ger_abbr_getvals TAB)
  returns a list of all values in the assoc-list TAB"
 (if (eq tab nil)
     nil
     (append (cdar tab) (ger_abbr_getvals (cdr tab)))))

(defvar german_currency
    (ger_abbr_getkeys ger_abbr_currency_tab)
; (ger_abbr_getvals ger_abbr_currency_tab))
  "a list of german waerungseinheiten created from ger_abbr_currency_tab")

(defvar german_months 
  (append (ger_abbr_getkeys ger_abbr_months_tab) (ger_abbr_getvals ger_abbr_months_tab))
  "a list of german month-names created from ger_abbr_months_tab")

(defvar german_days 
  (append (ger_abbr_getkeys ger_abbr_days_tab) (ger_abbr_getvals ger_abbr_days_tab))
  "a list of german day-names created from ger_abbr_days_tab")



(define (ger_lookup_comb_abbr name)
  "ger_lookup_comb_abbr NAME
   split and lookup combined abbr. like Vers.-Vert.-Vers. recursively"
  (if (string-matches name "[A-Za-z]*")
      (let ((uebersetzung (ger_abbr_lookup name ger_abbr_with_point_tab)))
	(if uebersetzung
	    uebersetzung
	    (german_parse_charlist name)))
      (let ((uebersetzung (ger_abbr_lookup (string-before name ".") ger_abbr_with_point_tab)))
	(append
	 (if uebersetzung
	     uebersetzung
	     (german_parse_charlist (string-before name ".")))
	 (ger_lookup_comb_abbr (string-after name "-"))))))

(define (ger_check_for_abbr token name)
  "(ger_check_for_abbr TOKEN NAME)
   checks heuristicaly if TOK is an abbreviation. It returns the expanded form if so, nil else.
   additionally it sets the token_pos-feature to 'abbr' for later modules"
  (cond
   ((string-matches name "[A-Za-z]")  ;;Token der Lnge eins sind Abkrzungen
    (item.set_feat token "token_pos" "abbr")
    (if (string-matches  (item.feat token "punc")  "\\..*")
	(item.set_feat token "punc" 
			     (string-after (item.feat token "punc") ".")))
    (list name)) ;token of lenth 1 are spoken as single letters.
   ((string-matches name "[BCDFGHJKLMNPQRSTVWXZbcdfghjklmnpqrstvwxz$]*") ;;Token, die nur aus Konsonanten 
    (item.set_feat token "token_pos" "abbr")                        ;;(ausser y) bestehen sind Abkrzungen
    (ger_translate_abbr token name))
   ;;Token mit Grossbuchstaben an einer Stelle ungleich der ersten sind eventuell Abkrzungen
   ;;Eigentlich sollte an dieser Stelle ine ausfuehrliche pruefung auf aussprechbarkeit
   ;;stattfinden. Das waere allerding sehr aufwendig. Bis jetzt werden  nur die Abkuerzungen erkannt, welche 
   ;;in einer Abkuerzungstabelle zu finden sind.
   ((string-matches name ".+[A-Z].*")                      
    (let ((exp (ger_abbr_lookup name ger_abbr_compl_tab)))  
      (if exp 
	  (begin 
	    (item.set_feat token "token_pos" "abbr")
	    exp)
	  nil)))))
;   ((string-matches name ".+\\..+")       ;;Token mit einem Punkt in der Mitte
;                                          ;;sind potent.  Abkrzungen?

   

(define (ger_translate_abbr token name)
  "(ger_translate_abbr TOKEN NAME)
   expands an abbreviation according to the several abbreviation tabs and throws away eventually existing '.'"
  (if (string-matches (item.feat token "punc") "\\..*")
      (let ((uebersetzung (ger_abbr_lookup name ger_abbr_at_eos_tab)))
	;;if abbr usually occurs at end of sentence let punctuation unchanged
	(if uebersetzung 
	    uebersetzung
	    (begin 
	      (set! uebersetzung (ger_abbr_lookup name ger_abbr_with_point_tab))
	      ;;if abbr doesn't usually occur at end of sentence delete '.' from punctuation
	      ;;if next token isn't a secure start of sentence
	      (if uebersetzung 
		  (begin
		    (if (not (member_string (item.feat token "n.name") 
					    german_satzanfang))
			(item.set_feat token "punc" 
					     (string-after 
					      (item.feat token "punc") ".")))
		    uebersetzung)
		  (begin
		    (set! uebersetzung (ger_abbr_lookup name ger_abbr_without_point_tab))
		    (if uebersetzung
			uebersetzung
			(begin
			  (set! uebersetzung (ger_abbr_lookup name ger_abbr_months_tab))
			  (if uebersetzung
			      uebersetzung
			      (begin
				(set! uebersetzung (ger_abbr_lookup name ger_abbr_days_tab))
				(if uebersetzung
				    uebersetzung
				    (begin
				      (set! uebersetzung (ger_abbr_lookup name ger_abbr_currency_tab))
				      (if uebersetzung
					  uebersetzung
					  (german_parse_charlist name)))))))))))))
      (let ((uebersetzung (ger_abbr_lookup name ger_abbr_without_point_tab)))
	(if uebersetzung
	    uebersetzung
	    (begin
	      (set! uebersetzung (ger_abbr_lookup name ger_abbr_months_tab))
	      (if uebersetzung
		  uebersetzung
		  (begin
		    (set! uebersetzung (ger_abbr_lookup name ger_abbr_days_tab))
		      (if uebersetzung
			  uebersetzung
			  (begin
			    (set! uebersetzung (ger_abbr_lookup name ger_abbr_currency_tab))
			    (if uebersetzung
				uebersetzung
				(german_parse_charlist name)))))))))))





(define (ger_tok_koenig sc)
  "(ger_tok_koenig sc)
Returns 1 if King like title is within 3 tokens before or 2 after."
  (if (or (member_string 
	   (downcase (item.feat sc "p.p.name"))
	   german_king_title)
	  (member_string 
	   (downcase (item.feat sc "p.p.p.name"))
	   german_king_title)
	  (member_string 
	   (downcase (item.feat sc "n.name"))
	   german_king_title)
	  (member_string
	   (downcase (item.feat sc "n.n.name"))
	   german_king_title))
      "1"
      "0"))

(define (ger_tok_koenigin sc)
  "(ger_tok_koenigin sc)
Returns 1 if Koenigin like title is within 3 tokens before or 2 after."
    (if (or (member_string 
	     (downcase (item.feat sc "p.p.name"))
	     german_queen_title)
	    (member_string 
	     (downcase (item.feat sc "p.p.p.name"))
	     german_queen_title)
	    (member_string 
	     (downcase (item.feat sc "n.name"))
	     german_queen_title)
	    (member_string
	     (downcase (item.feat sc "n.n.name"))
	     german_queen_title))
	"1"
	"0"))


(define (ger_tok_koenig_names sc)
  "(ger_tok_koenig_names sc)
Returns 1 if one token before is a King-like name."
  (if (or (member_string
       (downcase (item.feat sc 'p.name))
       german_king_name)
      (member_string
       (downcase (item.feat sc 'p.p.name))
       german_king_name))
      "1"
      "0"))

(define (ger_tok_koenigin_names sc)
  "(ger_tok_koenigin_names sc)
Returns 1 if one token before is a Koenigin-like name."
  (if (or (member_string
	   (downcase (item.feat sc 'p.name))
	   german_queen_name)
	  (member_string
	   (downcase (item.feat sc 'p.p.name))
	   german_queen_name))
      "1"
      "0"))

(provide 'ims_german_token_to_words_tools)




