;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*             Date   :  September 1997                                  */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/*  Functions used in identifying token-pos types (mainly years,         */ 
;/*  cardinals and currencies)                                            */
;/*                                                                       */
;/*=======================================================================*/


(require 'ims_german_token_to_words_lists)
(require 'ims_german_token_to_words_tools)


;(define (tok_string_as_letters name)
;  "(tok_string_as_letters NAME)
;Return list of letters marked as letter part of speech made
;by exploding NAME."
;  (mapcar
;   (lambda (letter)
;     (list (list 'name letter)
;	   (list 'pos token.letter_pos)))
;   (symbolexplode name)))

(define (ger_token_pos_guess sc)
"(ger_token_pos_guess SC)
Returns a general pos for SC's name. 
  numeric   (All digits),
  number    (float or comma'd numeric),
  sym       (Contains at least one non alphanumeric),
  month     (has month name (or abbrev)),
  day       (has day name (or abbrev)),
  sos       (is a start of sentence),
  rootname  (else downcased alphabetic).
Note this can be used to find token_pos but isn't used directly as
its not disciminatory enough."
  (let ((name (item.feat sc 'name)))
    (cond
     ((string-matches name "[0-9]+")
      'numeric)
     ((or (string-matches name "[0-9]+\\.[0-9]+")
	  (string-matches name 
	     "[0-9][0-9]?[0-9]?,\\([0-9][0-9][0-9],\\)*[0-9][0-9][0-9]"))
      'number)
     ((string-matches name ".*[^A-Za-z0-9].*")
      'sym)
     ((member_string name german_months) 
      'month)
     ((member_string name german_days)
      'day)
     ((member_string name german_satzanfang)      
      'sos) 
     (t 
      (downcase name)))))


;;;
;;;  Token pos are gross level part of speech tags which help decide
;;;  pronunciation of tokens (particular expansion of Tokens into words)
;;;  The most obvious example is identifying number types (ordinals,
;;;  years, digits or numbers).
;;;
;;; this Tree is build from hand. if somebody has a set of training data
;;; it can be built automatically (see chapter 14.3 "Homograph disambiguation"
;;; in "The Festival Speech Synthese System" Edition 1.2
(set! german_token_pos_cart_trees
  '(

    ;;  Format is (Regex Tree)
    ("[0-9]+" ;;digits only -> cardinal, ordinal or year?
     ((punc matches "\\..*") ;; number followed by a "."?
      ((n.lisp_ger_token_pos_guess is sos) 
       ;; nchstes Token Satzanfang -> Cardinal
       ((p.name in lisp_german_year_indicator)
	;; Jahr oder Card?
	((year))
	((p.lisp_ger_token_pos_guess is month)
	 ((year))
	 ((pp.name is als)
	  ((year))
	  ((cardinal)))))
       ((n.name is 0) ;; end of utterance?
	((p.name in ,german_year_indicator)
	 ;; Jahr oder Cardinal?
	 ((year))
	 ((p.lisp_ger_token_pos_guess is month)
	  ((year))
	  ((pp.name is als)
	   ((year))
	   ((cardinal)))))
	((p.lisp_ger_token_pos_guess is month)
	 ((year))
	 ((p.name in (Jahr Jahre Ende))
	  ((year))
	  ((ordinal))))))
      ((p.name in ,german_year_indicator)
       ;; Jahr oder Cardinal?
       ((year))
       ((p.lisp_ger_token_pos_guess is month)
	((year))
	((pp.name is als)
	  ((year))
	  ((cardinal)))))))
    
    ("[1-9][0-9]*\\,[0-9][0-9]"    ;; potentieller Kandidat 
     ((n.name in ,german_currency)        ;; fr Geldbetrag
      ((money))
      ((dezimalbruch))))

  ))




(provide 'ims_tokenpos)




