;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*             Date   :  Juni 1998                                       */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file contains additional scheme functions which are not defined  */
;/* in Festival SIOD.                                                     */
;/*                                                                       */
;/*=======================================================================*/

(define (string-replace str a b)
  "(string-replace STR A B) replace in string STR the substring A by B"
  (if (string-matches str (format nil ".*%s.*" a))
      (string-append 
       (string-before str a) b (string-replace (string-after str a) a b))
      str))
  
(define (sublis l exp)
  "e.g. (sublis '((a . b)) exp) 
   returns exp where all a's are substituted with b's"
  (if (cons? exp)
      (cons (sublis l (car exp))
	    (sublis l (cdr exp)))
      (let ((cell (assq exp l)))
	(if cell (cdr cell) exp))))

(define (subseq liste start end)
  "(subseq liste start end) returns the subsequence of liste 
   from position start to end"
  (let ((l liste)
	(erg-list nil)
	(pos 0))
    (while (< pos start)
	   (set! l (rest l))
	   (set! pos (+ pos 1)))
    (while (and (not (equal? pos end)) (not (equal? l nil)))
	  (set! erg-list (append erg-list (list (car l))))
	  (set! l (rest l))
	  (set! pos (+ pos 1)))
    erg-list))

(define (position x list start)
  "(position x list start) returns the first position where x occurs in list.
   Starting at position start."
  (let ((l list)
        (pos 0))
    (while (< pos start)
	   (set! l (rest l))
	   (set! pos (+ pos 1)))
    (while (and (not (equal? (first l) x)) (not (equal? l nil)))
	   (set! l (rest l))
	   (set! pos (+ pos 1)))
    (if (> pos (- (length list) 1)) 
	   nil
	   pos)))
    
            

(define (first x) 
"Returns the first element from ``x''."
  (car x))

(define (second x) 
"Returns the second element from ``x''."
  (cadr x))

(define (third x) 
"Returns the third element from ``x''."
  (caddr x))

(define (rest x) 
"Returns ``x'' without the first element."
  (cdr x))

(define (cons? x) 
"Checks, whether ``x'' is an atom (-> f) or (-> t)."
  (not (atom x)))


;(trace cons? second member first second rest position)

;; conversion list to string
(define (list->string l)
"Converts the list ``l'' into a string by using ``string-append''."
  (if (equal? (car l) nil) 
      (set! str '"")
      (string-append (car l) (list->string (cdr l)))))

;; conversion list to symbol
(define (list->symbol l)
  "(list->symbol L) returns the symbol derived from the concatenation 
   of the printnames of the members of L"
  (if (equal? (cdr l) nil) 
      (car l)
      (symbolconc (car l) (list->symbol (cdr l)))))

;; conversion symbol to list
(define (symbol->list s)
"Converts the symbol ``s'' into a list
by using ``symbolexplode''. Each character in ``s'' becomes an atom in this
list."
  (symbolexplode s))

;; conversion string to list
(define (string->list s)
"Converts the string ``s'' into a list
by using ``symbolexplode''. Each character in ``s'' becomes an atom in this
list."
  (symbolexplode s))

;; conversion symbol to string
(define (symbol->string s)
"Converts the symbol ``s'' into a string by using ``string-append''."
  (string-append s))


;; first char of a string
(define (string-car s)
"Returns the first character of the string ``s'' as a string."
  (string-append
    (car
     (symbolexplode s))))

;; all but first char of a string
(define (string-cdr s)
"Returns all characters from the string ``s'' besides the first."
  (let ((sy  (cdr (symbolexplode s))))
    (if (eq? sy nil)
	""
	(string-append
	 (list->string sy)))))


;; derived string functions
(define (string-cddr s)
"Returns all characters from the string ``s'' besides the
first two characters."
  (string-cdr (string-cdr s)))

(define (string-cdddr s)
"Returns all characters from the string ``s'' besides the
first three characters."
  (string-cdr (string-cddr s)))

(define (string-cadr s)
"Returns the second character from the string ``s''."
  (string-car (string-cdr s)))

(define (trunc number)
  "(trunc number) returns the integer-part of number."
  (parse-number (string-before (symbol->string number) ".")))


(define (removerec item list)
"(removerec ITEM LIST)
(Non-destructively) remove recursively ITEM from LIST."
  (cond    
   ((null list) nil)
   ((eq? item (car list))
    (removerec item (cdr list)))
   (t
    (cons (car list) (removerec item (cdr list))))))


(define (strcdr str)
  "(strcdr str) returns all but the first word from the string str.
   The first word is determined with the help of ``read-from-string''."
  (substring str 
	     (+ 1 (length (symbol->string (read-from-string str))))
	     (length str)))

(define (strcar str)
  "(strcar str) returns the first word from the string str
   with the help of ``read-from-string''"
  (read-from-string str))

(define (marks-substring str start end)
  "(marks-substring str start end) destructive! Returns the substring of str
   beginning with position start and ending with position end of str."
  (let ((ergstr "")
	(pos 0))
    (if (< start 0) (set! start 0))
    (if (> end (length str)) (set! end (length str)))
    (while (not (eq pos start))
	   (set! pos (+ pos 1))
	   (set! str (string-cdr str)))

    (while (< pos end)
	   (set! ergstr (string-append ergstr (string-car str)))
	   (set! str (string-cdr str))
	   (set! pos (+ pos 1)))
    ergstr))

(define (marks-read-from-string str)
  "(marks-read-from-string str) returns the words in str as a list of
  symbols."
  (if (> (length str)  0)
      (cons (read-from-string str) 
	    (marks-read-from-string (strcdr str)))
      nil))
    
(define (marks-string-to-list str)
"Converts a string containing less than 1000 characters
into a list. "
  (if (string-equal str "")
      nil
      (cons (read-from-string (marks-substring str 0 1))
	    (marks-string-to-list (marks-substring str 1 1000)))))

(define (lister n)
"Returns a list from n-1 downto 0."
  (if (equal? n 0) nil
      (cons (- n 1) (lister (- n 1)))))

(define (item.daughter_in_relation i rel_name)
  "(item.daughter_in_relation i rel_name)
    returns the daughter of i, which is in relation rel_name, 
    nil if there isn't any."
  (let ((d (item.daughters i)))
    (while (and d (not (item.relation (car d) rel_name)))
	   (set! d (cdr d)))
    (if d (car d) nil)))


(provide 'ims_siodfuncs)





