;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Gregor Mhler (moehler@ims.uni-stuttgart.de)    */
;/*             Date   :  ?                                               */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* Additional features for various relations (Phrase, Token, Word,       */
;/* Syllable) that are used at IMS.                                       */
;/* Some features reuqire special (non-OpenSource) modules to work.       */
;/* See the appendix with a list of features.                             */
;/*                                                                       */
;/*=======================================================================*/

(define (feat.exist item feat)
  "(feat.exist item feat) Returns true if the feature FEAT is not zero (GM)."
  (let ((f (item.feat item feat)))
    (not (or (equal? 0 f)
	     (equal? "0" f)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Phrase
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def_feature_docstring 'Phrase.lisp_phrase_num_syls
  "Phrase.lisp_num_syls
  Returns the number of syllables in a phrase as defined by the phrase relation. (GM)")
(define (phrase_num_syls phrase)
  (let ((sylcount 0))
    (mapcar
     (lambda (w)
       (let ((wrd (item.relation w 'Word)))
	 (mapcar
	  (lambda (s)
	   (set! sylcount(+ 1 sylcount)) )
	  (item.relation.daughters wrd 'SylStructure))))
     (item.daughters phrase))
    sylcount))

(def_feature_docstring 'Phrase.lisp_phrasepos_in_utt
  "Phrase.lisp_phrasepos_in_utt
  Returns the position of the phrase in the utterance (0 for first and 1 for last). (GM)")
(define (phrasepos_in_utt phrase)
  (let ((count -1)
	(pos -1)
	(ph (utt.relation.first (item.get_utt phrase) 'Phrase))) ; first phrase
    (while ph
       (set! count(+ 1 count))
       (if (equal? (item.feat phrase 'addr) (item.feat ph 'addr)) 
	   (set! pos count))
       (set! ph (item.next ph)))
    (cond
     ((equal? count 0) 0)
     (t
      (/ pos count)))))

(def_feature_docstring 'Phrase.lisp_phrase_duration
  "Phrase.lisp_phrase_duration
    Returns the duration of the phrase (without leading silence). (GM)")
(define (phrase_duration utt phrase)
  (- (item.feat phrase 'end)
     (item.feat phrase 'daughter1.R:Word.word_start)))


;;; tobi intonation

(def_feature_docstring 'Phrase.lisp_phrase_num_accents
  "Phrase.lisp_phrase_num_accents
  Returns the number of accents in the phrase. (GM)")
(define (phrase_num_accents phrase)
  (let ((count 0))
    (mapcar
     (lambda (w)
       (mapcar
	(lambda (s)
	  (if (and(not(equal? "NONE" (item.feat s 'tobi_accent)))
		  (not(equal? "UNKNOWN" (item.feat s 'tobi_accent))))
	      (set! count(+ 1 count))))
       (item.relation.daughters w 'SylStructure)))
       (item.daughters phrase))
  count))
 
(def_feature_docstring 'Phrase.lisp_tobi_low_boundary
  "Phrase.lisp_tobi_low_boundary
    Returns 1 if the phrase has an low L-L% or L% boundary, 0 otherwise. (GM)")
(define (tobi_low_boundary  phrase)
  (let ((final_syl (item.relation.daughtern (item.daughtern phrase) 'SylStructure)))

    (cond 
     ((equal? "L-L%"  (item.feat final_syl 'tobi_endtone)) 1)
     ((equal? "L%"  (item.feat final_syl 'tobi_endtone)) 1)
     (t 0))))

(def_feature_docstring 'Phrase.lisp_tobi_high_boundary
  "Phrase.lisp_tobi_high_boundary
    Returns 1 if the phrase has a high H-H% or H% boundary, 0 otherwise. (GM)")
(define (tobi_high_boundary  phrase)
   (let ((final_syl (item.relation.daughtern (item.daughtern phrase) 'SylStructure)))
   (print (item.feat final_syl 'tobi_endtone))
    (cond 
     ((equal? "H-H%"  (item.feat final_syl 'tobi_endtone)) 1)
     ((equal? "H%"  (item.feat final_syl 'tobi_endtone)) 1)
     (t 0))))

(def_feature_docstring 'Phrase.lisp_phrase_wordlist
  "Phrase.lisp_wordlist
   returns the list of words contained in the phrase (GM)")
(define (phrase_wordlist phrase)
  (if (symbol-bound? 'word_delim)
      nil (set! word_delim " "))
  (let ((w_str "("))
    (mapcar
     (lambda (w)
       (set! w_str (format nil "%s%s%s" w_str word_delim (item.feat w 'name))))
     (item.daughters phrase))
    (format nil "%s )%s" w_str word_delim)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Token
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def_feature_docstring 'Token.lisp_punc
  "Token.lisp_punc
   Returns relevant punctuation signs in words (to use them with wagon). (GM)")
(define (punc tok)
  (cond 
   ((equal? "," (item.feat tok 'punc))
    "comma")
   ((equal? "." (item.feat tok 'punc))
    "fullstop")
   ((equal? "?" (item.feat tok 'punc))
    "questmark")
   ((equal? "!" (item.feat tok 'punc))
    "exclmark")
   ((equal? ";" (item.feat tok 'punc))
    "semikolon")
   ((equal? ":" (item.feat tok 'punc))
    "colon")
   (t
    "0")))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Word
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; transcription

(def_feature_docstring 'Word.lisp_word_transcription
  "Returns the transcription of the word and 0 if no segments 
  belong to the syllable. (GM)")
(define (word_transcription wrd)
  (let ((trans ""))
  (mapcar
   (lambda (syl)
     (set! trans (string-append trans (item.feat syl 'lisp_syl_transcription))))
    (item.relation.daughters wrd 'SylStructure))
  (if (equal? trans "") "0" trans)))

(def_feature_docstring 'Word.lisp_with_accent
  "Word.lisp_with_accent. Do we accept an accent on this word? 
   No accent if this word was converted by LTS rules. (GM)")
(define (with_accent wrd)
  (cond 
   ((cadr(lex.lookup (item.feat wrd 'name))) ; POS not nil -> from lexicon
    "1")
   ((< (parse-number (item.feat wrd 'word_numsyls)) 3) ; o.k. with short LTS words
    "1")
   (t
    "0")))

(def_feature_docstring 'Word.lisp_contentw
  "Word.lisp_contentw. Is one if the word is a content word 
  (Verb, Noun, Cardinal, Adjective, FM). (AS)")
(define (contentw wrd)
  (cond 
   ((string-matches (item.feat wrd 'pos) "V.*") 1)
   ((string-matches (item.feat wrd 'pos) "N.*") 1)
   ((string-matches (item.feat wrd 'pos) "CARD") 1)
   ((string-matches (item.feat wrd 'pos) "FM") 1)
   ((string-matches (item.feat wrd 'pos) "ADJ.*") 1)
   (t 0)))

(def_feature_docstring 'Word.lisp_word_punc
  "Word.lisp_word_punc. Punctuation after token related to this word.
   0 if the following word belongs to the same token 
   (e.g. for expanded numbers). (GM)")
(define (word_punc wrd)
  (if (equal? (item.feat wrd 'R:Token.parent.addr)
	      (item.feat wrd 'n.R:Token.parent.addr))
      "0"
      (item.feat wrd 'R:Token.parent.punc)))

(def_feature_docstring 'Word.lisp_ustts_pos
  "Gives back the simpler tags from the celex tagset, 
   stts tags are maped using sttstags_celextags_tab (GM)")
(define (ustts_pos wrd)
  (let ((tag (item.feat wrd 'pos)))
    (cond
     ((equal? "stts" (Param.get 'pos_tagset))
      (cdr (assoc tag stts_ustts_postab)))
     (t tag))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Syllable
;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; transcription

(def_feature_docstring 'Syllable.lisp_syl_transcription
  "Returns the transcription of the syllable and 0 if no segments 
  belong to the syllable. (GM)")
(define (syl_transcription syl)
  (let ((trans ""))
  (mapcar
   (lambda (seg)
     (set! trans (string-append trans (item.feat seg 'name))))
     (item.daughters (item.relation syl 'SylStructure)))
  (if (equal? trans "") "0" trans)))

;;; syllable structure - voicing

(def_feature_docstring 'Syllable.lisp_voicing_start
  "Syllable.lisp_voicing_start
   gives back the beginning of voicing of a syllable. (GM)")
(define (voicing_start syl)
  (cond
   ((null syl) nil)
   (t (let ((segs(item.relation.daughters syl "SylStructure")))
	 (item.feat (first_voiced segs) "segment_start")))))

(def_feature_docstring 'Syllable.lisp_voicing_end
  "Syllable.lisp_voicing_end
   gives back the end of voicing of a syllable. (GM)")
(define (voicing_end syl)
  (cond
   ((null syl) nil)
   (t (let ((segs (item.relation.daughters syl "SylStructure")))
	 (item.feat (first_voiced (reverse segs)) "end")))))

(def_feature_docstring 'Syllable.lisp_first_voiced 
  "Returns first segment that is voiced (vowel or voiced consonant)
returns last segment if all are unvoiced. (GM)")
(define (first_voiced segs)
  (cond
   ((null (cdr segs))
    (car segs))  ;; last possibility
   ((equal? "+" (item.feat (car segs) "ph_vc"))
    (car segs))
   ((equal? "+" (item.feat (car segs) "ph_cvox"))
    (car segs))
   (t
    (first_voiced (cdr segs)))))

;;; syllable structure - temporal aspects (partially from tilt.scm)

(def_feature_docstring 'Syllable.lisp_syl_srhyme_start
  "Syllable.lisp_syl_srhyme_start
  Get the start of the s-rhyme, which is the beginning of the first 
  non-syllable-initial sonorant of the syllable, 
  cf. van Santen/Moebius, Moehler (Diss) (GM)")
(define (syl_srhyme_start syl)
  (let ((sttime (item.feat syl 'syllable_start)))
    (mapcar
     (lambda (s)
;       (format t "%s %s %s %f\n" 
;	       (item.feat s 'name)
;	       (item.feat s 'ph_vc)
;	       (item.feat s 'ph_ctype) sttime)
       (if (string-equal '+ (item.feat s 'ph_vc))
	   (if (or(string-equal 'n (item.feat s 'p.ph_ctype)) ; sonorant=nasal or liquid
		  (string-equal 'l (item.feat s 'p.ph_ctype)))
	       (set! sttime (item.feat s 'p.segment_start))
	       (set! sttime (item.feat s 'segment_start)))))
     (item.relation.daughters syl 'SylStructure))
    (if (<= sttime (item.feat syl 'syllable_start))
	 (item.feat syl 'syl_vowel_start) sttime)))

(def_feature_docstring 'Syllable.lisp_get_onset_length
  "Syllable.lisp_get_onset_length
  Length from start of syllable to start of vowel (tilt.scm untested).")
(define (get_onset_length syl)
  (cond
   ((< (- (syl_vowel_start syl)
	  (item.feat syl 'syllable_start))
       0.000)
    0.000)  ;; just in case
   (t
    (- (syl_vowel_start syl)
       (item.feat syl 'syllable_start)))))

(def_feature_docstring 'Syllable.lisp_get_rhyme_length
  "Syllable.lisp_get_rhyme_length
  Length from start of the vowel to end of syllable (tilt.scm untested).")
(define (get_rhyme_length syl)
  (- (item.feat syl 'syllable_end)
     (syl_vowel_start syl)))

(def_feature_docstring 'Syllable.lisp_vowel_class
  "Syllable.lisp_vowel_class
  H for high vowels, L for low vowels, rest is M (tilt.scm untested).")
(define (vowel_class syl)
  (let ((vh (phone_feature (item.feat syl 'syl_vowel) 'vheight)))
    (cond
     ((string-equal "1" vh) "H")
     ((string-equal "3" vh) "L")
     (t "M"))))

(def_feature_docstring 'Syllable.lisp_syl_vowel_start
  "Syllable.lisp_syl_vowel_start
  The start time of the vowel in this syllable (tilt.scm untested).")
(define (syl_vowel_start syl)
"(syl_vowel_start syl)
Start time of vowel in syl."
  (let ((sttime (item.feat syl 'syllable_start)))
    (mapcar
     (lambda (s)
       (if (string-equal '+ (item.feat s 'ph_vc))
	   (set! sttime (item.feat s 'segment_start))))
     (item.relation.daughters syl 'SylStructure))
    sttime))

(def_feature_docstring 'Syllable.lisp_syl_vowel_end
  "Syllable.lisp_syl_vowel_end
  The end time of the vowel in this syllable. (GM)")
(define (syl_vowel_end syl)
"(syl_vowel_start utt syl)
Start time of vowel in syl."
  (let ((sttime (item.feat syl 'syllable_start)))
    (mapcar
     (lambda (s)
       (if (string-equal '+ (item.feat s 'ph_vc))
	   (set! etime (item.feat s 'segment_end))))
     (item.relation.daughters syl 'SylStructure))
    etime))

;;; intonation

(def_feature_docstring 'Syllable.lisp_prev_accent
  "Syllable.lisp_prev_accent
   returns previous accent (no boundaries, just real accents!. (AS)")
(define (prev_accent syl)  ; AWB's last_accent_type also returns boundaries
  (if (equal? (item.feat syl 'tobi_accent) "NONE") 
      (if (or                            ;; syllable is not accented
	   (null (item.prev syl)) 
	   (null (equal? (item.feat syl 'p.tobi_endtone) "NONE")))
	  "NONE"                         ; no preceding syllables within ip
	  (prev_accent (item.prev syl))) ; check preceding syllables
      (item.feat syl 'tobi_accent)))

;;; intonation - tobi

(def_feature_docstring 'Syllable.lisp_tobi_accent_class
  "Syllable.lisp_tobi_accent_class
  Returns the the class of the ToBI accent on this syllable and "0" if there is none. (GM)")
(define (tobi_accent_class syl)
  (let ((tacc (item.feat syl 'tobi_accent)))
    (cond 
     ; for all languages
     ((equal? tacc "NONE")
      "0")
     ; German 
     ((equal? (Param.get 'Language) "german")
      (cond 
       ((member tacc '("HH*L" "H*M" "L*" "!H*" "*?" "L*H?" "H*L?" "L*HL?" "HH*L?" "H*M?" "H*?" "L*?" "L*!H?" "!H*L?")) ; rest class
	"*?")
       ((member tacc '("..L" "..H" "..L?" "..H?"))
	"0") ; not a real accent
       (t
	tacc)))
     ; American English
     (t
      (cond
       ((equal? tacc "H+!H*")
	"H*")
       ((equal? tacc "L*+!H")
	"L*+H")
;      ((equal? tacc "L+!H*")
;      "L+H*")
       ((equal? tacc "X*?")
	"*?")
       (t
	tacc))))))

(def_feature_docstring 'Syllable.lisp_tobi_endtone_class
  "Syllable.lisp_tobi_endtone_class
  Returns the class of the ToBI endtone on this syllable and 0 if there is none. (GM)")
(define (tobi_endtone_class syl)
  (let ((tendt (item.feat syl 'tobi_endtone)))
    (cond 
     ; for all languages
     ((equal? tendt "NONE")
      "0")
     ; German 
     ((equal? (Param.get 'Language) 'german)
      (cond 
       ((member tendt '("-" "%?" "%H" "%r" "L%?" "H%?" "-?"))
	"0")
       (t
	tendt)))
     ; AE: mapping because of low frequencies
     (t
      (cond
       ((member tendt '("H-H%" "H-L%" "H-" "!H-H%" "H-!H%"))
	"H%")
       ((member tendt '("!H-L%" "!H-" "%H")) ; not really high and %H
	"0")
       ((equal? tendt "L-!H%")
	"L-H%")
       ((member tendt '("X%?" "X-?" "%?"))
	"0")
       (t
	tendt))))))

(def_feature_docstring 'Syllable.lisp_tobi_basicaccent
  "Syllable.lisp_tobi_basicaccent
  only H*L and L*H accents, others are mapped to these 2 types or NONE. (GM)")
(define (tobi_basicaccent syl)
  (let ((tacc (item.feat syl 'tobi_accent)))
    (cond
     ((member tacc '("L*H" "L*!H" "L*HL" "L*H?" "L*!H?" "L*HL?"))
      "L*H")
     ((member tacc '("H*L" "!H*L" "HH*L" "H*" "H*L?" "!H*L?" "HH*L?" "H*?"))
      "H*L")
   (t
    "NONE"))))

(def_feature_docstring 'Syllable.lisp_tobi_basicbound
  "Syllable.lisp_tobi_basicbound
  basic boundaries: H%, %. L% is mapped on %. (GM)")
(define (tobi_basicbound syl)
  (let ((tendt (item.feat syl 'tobi_endtone)))
    (cond
     ((member tendt '("%" "L%" "%?" "L%?"))
      "%")
     ((member tendt '("H%" "H%?" ))
      "H%")
     (t
      "NONE"))))

(def_feature_docstring 'Syllable.lisp_tobi_simpleaccent
  "Syllable.lisp_tobi_simpleaccent
  just for the placement of accents: X* for accent NONE for no-acccent. (GM)")
(define (tobi_simpleaccent syl)
  (cond
   ((equal? "NONE" (item.feat syl 'tobi_accent))
    "NONE")
   (t
    "X*")))

(def_feature_docstring 'Syllable.lisp_tobi_simplebound
  "Syllable.lisp_tobi_simplebound
  simple boundaries: H% for high bounds, X% for other bounds and NONE for rest. (GM)")
(define (tobi_simplebound syl)
  (cond
   ((or(equal? "%"  (item.feat syl 'tobi_endtone))
       (equal? "L%" (item.feat syl 'tobi_endtone)))
    "X%")
   ((equal? "H%" (item.feat syl 'tobi_endtone))
    "H%")
   (t
    "NONE")))

(def_feature_docstring 'Syllable.lisp_tobi_early
  "Syllable.tobi_early
   Returns 1 if an early ToBI accent (currently: L+H*, H+!H*, HH*L)
   is marked on this syllable. (GM)")
(define (tobi_early syl)
  (cond
   ((or (equal? "L+H*" (item.feat syl 'tobi_accent))
	(equal? "H+!H*" (item.feat syl 'tobi_accent))
	(equal? "HH*L" (item.feat syl 'tobi_accent)))
    "1")
   (t "0")))

(def_feature_docstring 'Syllable.lisp_tobi_accented
  "Syllable.lisp_tobi_accented
  a slight modification of the Syllable.accented feature: 
  the syl is accented when a tobi accent is marked
  (Syllable.accented would be 1 when a boundary alone occurs). (GM)")
(define (tobi_accented syl)
  (cond
   ((equal? "NONE" (item.feat syl 'tobi_accent))
    0)
   (t
    1)))

(def_feature_docstring 'Syllable.lisp_tobi_initbound
  "Syllable.tobi_early
   Returns 1 if a ToBI initial boundary (currently: %H)
   is marked on this syllable. (GM)")
(define (tobi_initbound syl)
  (cond
   ((or (equal? "%H" (item.feat syl 'tobi_accent))
	)
    "1")
   (t "0")))

;;; intonation - tobi - looking left

(def_feature_docstring 'Syllable.lisp_prev_IntEvent_H
  "Syllable.lisp_prev_IntEvent_H
  Returns "1" if previous intonation event starts with a high tone. (GM)")
(define (prev_IntEvent_H syl)
  (let ((pie (item.feat syl 'lisp_prev_IntEvent)))
    (cond
     ((member pie '("H*" "L*+H" "L+H*" "H-" "H-H%" "L*H" "H%" "..H")) 
      "1")
     (t "0"))))

(def_feature_docstring 'Syllable.lisp_prev_IntEvent
  "Syllable.lisp_prev_IntEvent
  Returns prev intonation event. Boundaries are considered to be
  to the right of accents within the same syllable. (GM)")
(define (prev_IntEvent syl)
    (cond 
     ((not syl) "0") ; first in utt
     (t
      (car(cont_prev_IntEvent (item.prev syl) 1)))))

(def_feature_docstring 'Syllable.lisp_dist_prev_IntEvent
  "Syllable.lisp_dist_prev_IntEvent
  Returns distance in syllable to prev intonation event. 
  If there is none the distance to utterance start is returned")
(define (dist_prev_IntEvent syl)
  (cond 
   ((not syl) 0) ; first in utt
   (t
    (cadr(cont_prev_IntEvent (item.prev syl) 1)))))

;; the basic left-looking function
(define (cont_prev_IntEvent syl count)
  "returns type and distance of prev tobi accent beginning in this syl"
    (cond 
     ((not syl) (list "0" count)) ; first in utt
     ((not(equal? "NONE" (item.feat syl 'tobi_endtone))) ; endtone
      (list (item.feat syl 'tobi_endtone) count))
     ((not(equal? "NONE" (item.feat syl 'tobi_accent))) ; accent
      (list (item.feat syl 'tobi_accent) count))
     (t
      (cont_prev_IntEvent (item.prev syl) (+ 1 count)))))

;;; intonation - tobi - looking right

(def_feature_docstring 'Syllable.lisp_next_IntEvent_H
  "Syllable.lisp_next_IntEvent_H
  Returns 1 if next intonation event starts with a high tone. (GM)")
(define (next_IntEvent_H syl)
  (let ((pie (item.feat syl 'lisp_next_IntEvent)))
    (cond
     ((member pie '("H*" "H+!H*" "H-" "H-H%" "H*L" "HH*L" "H*M" "H%" "%H" "..H"))
      "1")
     (t "0"))))

(def_feature_docstring 'Syllable.lisp_next_IntEvent
  "Syllable.lisp_next_IntEvent
  Returns next intonation event. Boundaries are considered to be
  to the right of accents within the same syllable. (GM)")
(define (next_IntEvent syl)
  (cond 
     ((not syl) 0) ; should not happen
     ((not(equal? "NONE" (item.feat syl 'tobi_endtone))) ; endtone in this syl
      (item.feat syl 'tobi_endtone))
     (t
      (car (cont_next_IntEvent (item.next syl) 1)))))

(def_feature_docstring 'Syllable.lisp_dist_next_IntEvent
  "Syllable.lisp_next_IntEvent
  Returns next intonation event. It is the accent 
  if same syllable holds a boundary as well. (GM)")
(define (dist_next_IntEvent syl)
  (cond 
     ((not syl) 0) ; should not happen
     ((not(equal? "NONE" (item.feat syl 'tobi_endtone))) ; endtone in this syl
      0)
     (t
      (cadr (cont_next_IntEvent (item.next syl) 1)))))

(define (cont_next_IntEvent syl count)
    (cond 
     ((not syl) (list "0" count)) ; should not happen
     ((not(equal? "NONE" (item.feat syl 'tobi_accent))) ; accent
      (list (item.feat syl 'tobi_accent) count))
     ((not(equal? "NONE" (item.feat syl 'tobi_endtone))) ; endtone
      (list (item.feat syl 'tobi_endtone) count))
     (t
      (cont_next_IntEvent (item.next syl) (+ 1 count)))))


;;; global prosodic features

(def_feature_docstring 'Syllable.lisp_position_in_phrase
  "Syllable.lisp_position_in_phrase
  Returns the position of the syl in the phrase (0 is initial, 1 is final). (GM)")
(define (position_in_phrase syl)
  (let ((ph_num_syl 
	 (item.feat syl 'R:SylStructure.parent.R:Phrase.parent.lisp_phrase_num_syls)))
    (cond
     ((eq ph_num_syl 0) 0)
     ((eq ph_num_syl 1) 0)
     (t
      (/ (item.feat syl 'syl_in) (- ph_num_syl 1))))))


(def_feature_docstring 'Syllable.lisp_next_sil_duration
  "Syllable.lisp_next_sil_duration
   Returns the length of the next pause. (GM)")
(define (next_sil_duration syl)
  ; seg is the last segment of the current syl:
  (let ((seg (item.relation (item.relation.daughtern syl 'SylStructure) 'Segment)))
    
    (while (and (item.next seg)
	      (not (phone_is_silence (item.feat seg 'name))))
    	   (set! seg (item.next seg)))
    
    (if (phone_is_silence (item.feat seg 'name)) 
	(item.feat seg 'segment_duration)
	0))
)
      
;;; intonation - general (mostly from tilt.scm)

(def_feature_docstring 'Syllable.lisp_next_stress
  "Syllable.lisp_next_stress
  Number of syllables to next stressed syllable. 0 if this syllable is
  stressed.  It is effectively assumed the syllable after the last syllable
  is stressed (tilt.scm untested). (GM)")
(define (next_stress syl)
  (cond 
   ((null syl) 0)
   ((string-equal (item.feat syl 'stress) "1")
    0)
   (t
    (+ 1 (next_stress (item.relation.next syl 'Syllable))))))

(def_feature_docstring 'Syllable.lisp_last_stress
  "Syllable.lisp_last_stress
  Number of syllables from previous stressed syllable.  0 if this syllable
  is stressed.  It is effectively assumed that the syllable before the 
  first syllable is stressed (tilt.scm untested). (GM)")
(define (last_stress syl)
  (cond 
   ((null syl) 0)
   ((string-equal (item.feat syl 'stress) "1")
    0)
   (t
    (+ 1 (last_stress (item.relation.prev syl 'Syllable))))))


;;; others

(def_feature_docstring 'Syllable.lisp_syl_punc
  "Syllable.lisp_punc
   Returns the punctuation feature (in words) for the Token the Syllable belongs to,
   but only if it is the final syllable in the word. (GM)")
(define (syl_punc syl)
  (cond
   ((equal? (item.feat syl 'R:SylStructure.parent.addr) ; i.e. Word.addr
	    (item.feat syl 'n.R:SylStructure.parent.addr))
    "0")
   (t 
    (item.feat syl 'R:SylStructure.parent.R:Token.parent.lisp_punc)))) ; i.e. punc of token

;;; painte access through the syllable

(def_feature_docstring 'Syllable.lisp_painte_name
  "Syllable.lisp_painte_name
  Returns the name of the painte event related to this syllable 
  ('pfun' or 'meanf0'). (GM)")
(def_feature_docstring 'Syllable.lisp_painte_a1
  "Syllable.lisp_painte_a1
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_a2
  "Syllable.lisp_painte_a2
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_b
  "Syllable.lisp_painte_b
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_c1
  "Syllable.lisp_painte_c1
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_c2
  "Syllable.lisp_painte_c2
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_d
  "Syllable.lisp_painte_d
  Returns the value of the painte parameter related to this syllable. (GM)")
(def_feature_docstring 'Syllable.lisp_painte_cw
  "Syllable.lisp_painte_cw
  Returns the number of the codeword related to this syllable. The codebook
  size is determined by the parameter Painte_codebooksize. (GM)")

(define (painte_name syl)
  (item.feat syl 'R:ParamIntonation.daughter1.name))
(define (painte_a1 syl)
  (item.feat syl 'R:ParamIntonation.daughter1.a1))
(define (painte_a2 syl)
  (item.feat syl 'R:ParamIntonation.daughter1.a2))
(define (painte_b syl)
  (item.feat syl 'R:ParamIntonation.daughter1.b))
(define (painte_c1 syl)
  (item.feat syl 'R:ParamIntonation.daughter1.c1))
(define (painte_c2 syl)
  (item.feat syl 'R:ParamIntonation.daughter1.c2))
(define (painte_d syl)
  (item.feat syl 'R:ParamIntonation.daughter1.d))
(define (painte_cw syl)
  (let ((cw (item.feat syl(string-append "R:ParamIntonation.daughter1.cw"
					 (Param.get 'Painte_codebooksize)))))
    (if (equal? "pfun" (item.feat syl 'lisp_painte_name))
	cw "-1")))

; the following features only work if the painte parameters were determined 
; without syllable normalization
(def_feature_docstring 'Syllable.lisp_painte_risetime
  "Syllable.lisp_painte_risetime
  Returns the rise time of the painte event related to this syllable. (GM)")
(define (painte_risetime syl)
  (if (equal? "pfun" (item.feat syl 'lisp_painte_name)) 
      (map_painte_a (item.feat syl 'lisp_painte_a1))
      0))
(def_feature_docstring 'Syllable.lisp_painte_falltime
  "Syllable.lisp_painte_falltime
  Returns the fall time of the painte event related to this syllable. (GM)")
(define (painte_falltime syl)
  (if (equal? "pfun" (item.feat syl 'lisp_painte_name)) 
      (map_painte_a (item.feat syl 'lisp_painte_a2))
      0))
(def_feature_docstring 'Syllable.lisp_painte_peak_sylpos
  "Syllable.lisp_painte_peak_sylpos
  Returns the position of the peak within the syllable structure. 
  0 is start of syllable, 0.5 start of s-rhyme, 0.8 end of nucleus and 1 end of syl. (GM)")
(define (painte_peak_sylpos syl)
  (if (equal? "pfun" (item.feat syl 'lisp_painte_name)) 
      (map_painte_b syl (item.feat syl 'lisp_painte_b))
      0))

(provide 'ims_extra_features)
