;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Gregor Mhler (moehler@ims.uni-stuttgart.de)    */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This is the master file of the OpenSource version of IMS German       */
;/* Festival:                                                             */
;/* The path is extended to festival/lib/german which holds               */
;/* the German Scheme files, the required modules are imported and        */
;/* some original Festival functions are adapted to German.               */
;/*                                                                       */
;/*=======================================================================*/

(format t "German Festival 1.2-os (1.4.1):release August 2000\n")

;;set load-path to find the German scheme-libraries
(set! load-path (append   (list (string-append libdir "/german")) load-path)) 

(require 'ims_extra_features) 
(require 'ims_german_voices_opensource)

;; should finally go to languages.scm
(define (language_german)
"(language_german)
Set up language parameters for German."

  (set! female1 voice_german_de3_os)
  (set! female2 voice_german_de1_os)
  (set! male1   voice_german_de2_os)
  (set! voice_default voice_german_de2_os) 

  (voice_default)
  (Param.set 'Language 'german)
)

(define (select_language language)
  (cond
   ((or (string-equal language 'britishenglish)
	(string-equal language 'english))  ;; we all know its the *real* English
    (language_british_english))
   ((string-equal language 'americanenglish)
    (language_american_english))
   ((string-equal language 'scotsgaelic)
    (language_scots_gaelic))
   ((string-equal language 'welsh)
    (language_welsh))
   ((string-equal language 'spanish)
    (language_castillian_spanish))
   ((string-equal language 'klingon)
    (language_klingon))
   ((string-equal language 'german)
    (language_german))
   (t
    (print "Unsupported language, using German")
    (language_german))
  )
)


;; a German intro
(define (intro-german)
"(intro-german)
 Synthesize an introduction to the Festival Speech Synthesis System
 in German."
  (if (equal? (Param.get 'Language) "german") 
      () (language_german))
  (tts (string-append libdir "/../examples/germintro.text") nil)
)

;; changes to functions in festival/lib/mbrola.scm
;; this version uses a phone-map table to map sampa phones
;; that are not covered in the MBROLA database
(require 'mbrola)
(define (save_segments_mbrola utt filename)
  "(save_segments_mbrola UTT FILENAME)
  Save segment information in MBROLA format in filename.  The format is
  phone duration (ms) [% position F0 target]*. [see MBROLA]"
  (let ((fd (fopen filename "w")))
    (mapcar
     (lambda (segment) 
       (save_seg_mbrola_entry 
	(mbrola_map_phone (item.feat segment 'name))
	(item.feat segment 'segment_start)
	(item.feat segment 'segment_duration)
	(mapcar
	 (lambda (targ_item)
	   (list
	    (item.feat targ_item "pos")
	    (item.feat targ_item "f0")))
	 (item.relation.daughters segment 'Target)) ;; list of targets
	fd))
     (utt.relation.items utt 'Segment))
    (fclose fd)))

(defvar mbrola_phonemap_tab nil
"mapping between phones in internal festival representation to MBROLA phones.")

(set! mbrola_phonemap_tab
      '(("?" . "_"))) ; map glottal stops to pauses

(define (mbrola_map_phone name)
  "(mbrola_map_phone name)
   map some phones (like glottal stops) may from the 
   internal festival representation to MBROLA phones."
    (let ((mname(cdr (assoc name mbrola_phonemap_tab ))))
      (if mname mname name)))

(provide 'ims_german_opensource)












